-----------------------------------------
-- Addon: Relational DKP
-- By: Shazear 
-- Shizukana <Veterans of the Phoenix>, Malygos
-- e-mail: shazear@hotmail.com
-- Code also used from CTRT
-- References: 
-----------------------------------------
RDKP_SZ_HEADER = "Relational DKP System UI " .. RDKP_FULL_VERSION;
RDKP_SZ_SETTINGS = "Relational DKP Settings";

function RDKP_ShowUI()
	-- This is the Command to show the UI 
	RDKP_Debug("RDKP_ShowUI()");
	RelationalDKP_TopFrame:Show();
end

function RDKP_OnShow()
	RDKP_Debug("RDKP_OnShow()");
	RDKP_UpdateMasterPlayerList();
	RDKP_PriorityOnClick();
end

function RDKP_GetZones()
	local szZones = "";
	for i, v in RDKP_Table_Zones do
		szZones = szZones .. "\n" .. v
	end
	return szZones;
end 

function RDKP_ShowSummary()
	RDKP_Debug("RDKP_ShowSummary()");
	-- Show the summary of players, along w/ their priority value.
	-- Sort Player list by Priority
	RDKP_SortMasterListbyPriority();
	-- return that table - elements will need to be selectable.
	return RDKP_GetPlayers();
end

function RDKP_PopulatePriorityButtons()
	RDKP_Debug("RDKP_PopulateListButtons()");
	RelationalDKP_DataFrame_Text:SetText("");
	local num = 0;
	-- Check if we're in a raid:
	-- AND Check to see if we only want the current members of the Raid 
	-- If we're not in a raid it doesn't matter if the FilterBy is set, we want to show the full list.
	FauxScrollFrame_Update(RDKP_ScrollFrame,table.getn(RDKP_Table_PlayersSorted),5,RDKP_PRIORITY_FRAME_HEIGHT)
	local iPriorityListOffset = FauxScrollFrame_GetOffset(RDKP_ScrollFrame);
	if (CT_RaidTracker_GetCurrentRaid and RDKP_FilterBy.RaidMembers) then
		RDKP_UpdateRaidList();
		-- if so, then check to see if we only want those who will send tells (IE they want the current item up for bid)
		if (RDKP_FilterBy["PST"]) then
			-- Start accepting tells for bidding.  
			-- need to work on that hook
			RDKP_PriorityButtonsByFilter(RDKP_RaidTellList, iPriorityListOffset);
		else 
			RDKP_PriorityButtonsByFilter(RDKP_Table_RaidPlayers, iPriorityListOffset);
		end	
	elseif (RDKP_Table_MasterPlayers and RDKP_Table_PlayersSorted) then 
		RDKP_PriorityButtonsByFilter(nil, iPriorityListOffset);
	else
		RDKP_Debug("RDKP_Table_MasterPlayers == nil");
	end
end

function RDKP_PriorityButtonsByFilter(tFilter, iStartingKey)
	RDKP_Debug("RDKP_PriorityButtonsByFilter()");
	local num = 0;
	-- Setup Header info.
	RDKP_PriorityButtonHeader_PlayerName:SetText("Player:")
	RDKP_PriorityButtonHeader_PlayerDKP:SetText("DKP: / GR: ")
	RDKP_PriorityButtonHeader_PlayerPriority:SetText("Priority: ")
	-- First, clear all the existing Priority buttons:
	for i = 1, RDKP_PRIORITY_BUTTON_QTY, 1 do
		local szName = "RDKP_PriorityButton" .. i .. "_PlayerName";
		local szDKP = "RDKP_PriorityButton" .. i .. "_PlayerDKP";
		local szPriority = "RDKP_PriorityButton" .. i .. "_PlayerPriority";

		getglobal("RDKP_PriorityButton" .. i):Hide();
		getglobal(szName):SetText("");
		getglobal(szDKP):SetText("");
		getglobal(szPriority):SetText("");
	end
	-- Show all raid members sorted like the full guild list.
	for k, v in RDKP_Table_PlayersSorted do 
		if (k >= iStartingKey) then
			if (tFilter) then
				-- since the sorting will be the same as before, we can utilize that list.
				-- Need to filter the list by who's in the raid.
				if (RDKP_SearchList(tFilter, v)) then
					if (num < RDKP_PRIORITY_BUTTON_QTY) then 
						-- For Now only show the top 19.
						num = num + 1;
						local szName = "RDKP_PriorityButton" .. num .. "_PlayerName";
						local szDKP = "RDKP_PriorityButton" .. num .. "_PlayerDKP";
						local szPriority = "RDKP_PriorityButton" .. num .. "_PlayerPriority";

						getglobal("RDKP_PriorityButton" .. num):Show();
						getglobal(szName):SetText(v);
			 			getglobal(szDKP):SetText(RDKP_Table_MasterPlayers[v]["DKP" ] .. " / " .. RDKP_Table_MasterPlayers[v]["GR"]);
			 			getglobal(szPriority):SetText(RDKP_Table_MasterPlayers[v]["PRI" ]);
			 		else
			 			return;
					end 
				end
			else
				if (num < RDKP_PRIORITY_BUTTON_QTY) then 
					-- For Now only show the top 19.
					num = num + 1;
					local szName = "RDKP_PriorityButton" .. num .. "_PlayerName";
					local szDKP = "RDKP_PriorityButton" .. num .. "_PlayerDKP";
					local szPriority = "RDKP_PriorityButton" .. num .. "_PlayerPriority";
					getglobal(szName):SetText(v);
		 			getglobal(szDKP):SetText(RDKP_Table_MasterPlayers[v]["DKP" ] .. " / " .. RDKP_Table_MasterPlayers[v]["GR"]);
		 			getglobal(szPriority):SetText(RDKP_Table_MasterPlayers[v]["PRI" ]);
		 			if ((v ~= nil) and (v ~= "")) then
						getglobal("RDKP_PriorityButton" .. num):Show();
					else
						getglobal("RDKP_PriorityButton" .. num):Hide();
					end
		 		else
		 			return;
				end 
			end
		end
	end
end 

function RDKP_ShowPriorityButtons(bShow)
	if (bShow) then 
		RDKP_ShowSettings(false);
	end
	if bShow then
		RDKP_PriorityButtonHeader:Show();
		getglobal("RDKP_ScrollFrame"):Show();
		getglobal("FilterByRaidButton"):Show();
		getglobal("FilterByTellsButton"):Show();
		for num = 1, RDKP_PRIORITY_BUTTON_QTY, 1 do 
			getglobal("RDKP_PriorityButton" .. num):Show();
		end
	else
		RDKP_PriorityButtonHeader:Hide();
		getglobal("RDKP_MemberDetailFrame"):Hide();
		getglobal("RDKP_ScrollFrame"):Hide();
		getglobal("FilterByRaidButton"):Hide();
		getglobal("FilterByTellsButton"):Hide();
		for num = 1, RDKP_PRIORITY_BUTTON_QTY, 1 do 
			getglobal("RDKP_PriorityButton" .. num):Hide();
		end
	end
end

function RDKP_PopulateSettingsFrames()
	RDKP_Debug("RDKP_PopulateSettingsFrames()");
	RelationalDKP_DataFrame_Text:SetText("");
	RDKP_ConditionalDataReset();
	-- First we have 4 settings that are separate from the zone GR values:
	-- Header:
	getglobal("RelationalDKP_SettingFrameHeader_PlayerName"):SetText("DKP Type:");
	getglobal("RelationalDKP_SettingFrameHeader_PlayerDKP"):SetText("Award:");
	--getglobal("RelationalDKP_SettingFrameHeader_PlayerPriority"):SetText("");
	-- Values:
		-- Trash Mobs:
	getglobal("RelationalDKP_SettingFrameTrash_PlayerName"):SetText("Trash Mob Kills: ");
	getglobal("RelationalDKP_SettingFrameTrash_PlayerDKP"):SetText(RDKP_TABLE_DKP_VALUE.trash);
	--getglobal("RelationalDKP_SettingFrameTrash_PlayerPriority"):SetText("");
		-- Boss Kills:
	getglobal("RelationalDKP_SettingFrameBoss_PlayerName"):SetText("Boss Kills: ");
	getglobal("RelationalDKP_SettingFrameBoss_PlayerDKP"):SetText(RDKP_TABLE_DKP_VALUE.kill);
	--getglobal("RelationalDKP_SettingFrameBoss_PlayerPriority"):SetText("");
		-- DKP per Hour:
	getglobal("RelationalDKP_SettingFrameHour_PlayerName"):SetText("DKP per Hour: ");
	getglobal("RelationalDKP_SettingFrameHour_PlayerDKP"):SetText(RDKP_TABLE_DKP_VALUE.hour);
	--getglobal("RelationalDKP_SettingFrameHour_PlayerPriority"):SetText("");
		-- Start of Raid:
	getglobal("RelationalDKP_SettingFrameStart_PlayerName"):SetText("Not Yet Implemented:");
	getglobal("RelationalDKP_SettingFrameStart_PlayerDKP"):SetText(RDKP_TABLE_DKP_VALUE.raid_start);
	--getglobal("RelationalDKP_SettingFrameStart_PlayerPriority"):SetText("");

	-- Set Headers:
	getglobal("RelationalDKP_SettingFrame0_ZoneName"):SetText("Gear Value by Zone:");
	getglobal("RelationalDKP_SettingFrame0_UnCommonValue"):SetText("Greens:");
	getglobal("RelationalDKP_SettingFrame0_RareValue"):SetText("Rares:");
	getglobal("RelationalDKP_SettingFrame0_EpicValue"):SetText("Epics:");
	-- For each zone, display a settings frame:
	for iKey, szValue in RDKP_Table_Zones do
		if (iKey < 14) then 
			local szName = "RelationalDKP_SettingFrame" .. iKey .. "_ZoneName";
			local szUCGR = "RelationalDKP_SettingFrame" .. iKey .. "_UnCommonValue";
			local szRareGR = "RelationalDKP_SettingFrame" .. iKey .. "_RareValue";
			local szEpicGR = "RelationalDKP_SettingFrame" .. iKey .. "_EpicValue";

			RDKP_Debug(szValue); 
			getglobal(szName):SetText(szValue);
			if (RDKP_GR_VALUE) then
				if ( RDKP_GR_VALUE["ff1eff00"][szValue] ) then 
					getglobal(szUCGR):SetText(RDKP_GR_VALUE["ff1eff00"][szValue]);
				end
				if (RDKP_GR_VALUE["ff0070dd"][szValue]) then
					getglobal(szRareGR):SetText(RDKP_GR_VALUE["ff0070dd"][szValue]);
				end
				if ( RDKP_GR_VALUE["ffa335ee"][szValue] ) then
					getglobal(szEpicGR):SetText(RDKP_GR_VALUE["ffa335ee"][szValue]);
				end
			end
		end
	end
end

function RDKP_ShowSettings(bShow)
	if (bShow) then 
		RDKP_ShowPriorityButtons(false);
	end
	if bShow then
		if (RDKP_IsDKPOfficer()) then
			RDKP_SaveSettingsButton:Show();
		end
		RelationalDKP_SettingFrameHeader:Show();
		RelationalDKP_SettingFrameTrash:Show();
		RelationalDKP_SettingFrameBoss:Show();
		RelationalDKP_SettingFrameHour:Show();
		RelationalDKP_SettingFrameStart:Show();
		for num = 0, 13, 1 do 
			getglobal("RelationalDKP_SettingFrame" .. num):Show();
		end
	else
		RDKP_SaveSettingsButton:Hide();
		getglobal("RDKP_SettingDetailFrame"):Hide();
		RelationalDKP_SettingFrameHeader:Hide();
		RelationalDKP_SettingFrameTrash:Hide();
		RelationalDKP_SettingFrameBoss:Hide();
		RelationalDKP_SettingFrameHour:Hide();
		RelationalDKP_SettingFrameStart:Hide();
		for num = 0, 13, 1 do 
			getglobal("RelationalDKP_SettingFrame" .. num):Hide();
		end
	end
end

function RDKP_PriorityOnClick()
	RDKP_Debug("RDKP_PriorityOnClick()");
	RDKP_SortMasterListbyPriority();
	RDKP_PopulatePriorityButtons();
	RDKP_ShowSettings(false);
	RDKP_ShowPriorityButtons(true);
	if RDKP_IsDKPOfficer() then
		RDKP_GR_VALUE_TEMP = RDKP_GR_VALUE ;
		RDKP_TABLE_DKP_VALUE_TEMP = RDKP_TABLE_DKP_VALUE;
	end
end

function RDKP_SettingsOnClick()
	RDKP_Debug("RDKP_SettingsOnClick()");
	RDKP_PopulateSettingsFrames();
	RDKP_ShowPriorityButtons(false);
	RDKP_ShowSettings(true);
	-- On officers accounts, populate the temp settings table w/ the current values incase they make changes.
	if RDKP_IsDKPOfficer() then
		RDKP_GR_VALUE_TEMP = RDKP_GR_VALUE;
		RDKP_TABLE_DKP_VALUE_TEMP = RDKP_TABLE_DKP_VALUE;
	end
end

function RDKP_RaidStartOnClick()
	RDKP_Debug("RDKP_RaidStartOnClick()");
	if RDKP_IsDKPOfficer() then
		RDKP_GR_VALUE_TEMP = RDKP_GR_VALUE ;
		RDKP_TABLE_DKP_VALUE_TEMP = RDKP_TABLE_DKP_VALUE;
	end
	RDKP_ShowSettings(false);
	RDKP_ShowPriorityButtons(false);
	if (not (RDKP_Table_RaidInfo["RDKP_bCurrentRaid"])) then
		-- Starting raid.
		RDKP_StartRaid();
		RelationalDKP_TopFrame_StartRaid:SetText("End Raid");
		RelationalDKP_DataFrame_Text:SetText("Starting Raid...\nPlease click 'EndRaid' when the raid is finished.\nHowever, raid tracking/DKP will be closed if you reload your ui, so the raid will have to be restarted.\n However, if you leave the group, and there's a new Raid Leader, if he has this AddOn, then he should be able to resume DKP & Gear Awards.");
	elseif (RDKP_Table_RaidInfo["RDKP_bCurrentRaid"]) then
		-- Ending raid.
		RDKP_EndRaid();
		RelationalDKP_TopFrame_StartRaid:SetText("Start Raid");
		RelationalDKP_DataFrame_Text:SetText("Ending Raid...");
	end
end

function RDKP_SetAsMainOnClick()
	RDKP_Debug("RDKP_SetAsMainOnClick()");
	if RDKP_IsDKPOfficer() then
		RDKP_GR_VALUE_TEMP = RDKP_GR_VALUE ;
		RDKP_TABLE_DKP_VALUE_TEMP = RDKP_TABLE_DKP_VALUE;
	end
	RDKP_ShowPriorityButtons(false);
	RelationalDKP_DataFrame_Text:SetText("The current character is now the main for any others for this player.");
	RDKP_PlayerMap_SetAsMain();
end

function RDKP_PriorityButton_OnClick(arg1)
	RDKP_Debug("RDKP_PriorityButton_OnClick()");
	-- If you have DKP Officer rights
	if (RDKP_IsDKPOfficer() or RDKP_b_Debug) then
		-- display pop-up for adding DKP or GR to the character selected.
		RDKP_TableForAudit.PlayerName = getglobal(this:GetName() .. "_PlayerName"):GetText();
		-- If there's no name in the frame, then don't show the frame.:
		if (RDKP_TableForAudit.PlayerName) then
			RDKP_MemberDetailName:SetText(RDKP_TableForAudit.PlayerName);  
			RDKP_MemberDetailDKPText:SetText(RDKP_Table_MasterPlayers[RDKP_TableForAudit.PlayerName]["DKP"]);
			RDKP_MemberDetailGRText:SetText(RDKP_Table_MasterPlayers[RDKP_TableForAudit.PlayerName]["GR"]);
			RDKP_MemberDetailPRIText:SetText(RDKP_Table_MasterPlayers[RDKP_TableForAudit.PlayerName]["PRI"]);
			RDKP_MemberDetailFrame:Show();
		end
	end
end

function RDKP_DKPSettingButton_OnClick(arg1)
	RDKP_Debug("RDKP_DKPSettingButton_OnClick()");
	-- If you have DKP Officer rights
	if (RDKP_IsDKPOfficer() or RDKP_b_Debug) then
		-- display pop-up for adding DKP or GR to the character selected.
		RDKP_Setting_Selected = getglobal(this:GetName() .. "_PlayerName"):GetText();
		-- If there's no name in the frame, then don't show the frame.:
		if ((RDKP_Setting_Selected) and (RDKP_Setting_Selected ~= "DKP Type:")) then
			RDKP_Debug("RDKP_Setting_Selected = " .. RDKP_Setting_Selected);
			StaticPopup_Show("ADJUST_DKP_SETTING");
		end
	end
end

function RDKP_GRSettingButton_OnClick(arg1)
	RDKP_Debug("RDKP_DKPSettingButton_OnClick()");
	-- If you have DKP Officer rights
	if (RDKP_IsDKPOfficer() or RDKP_b_Debug) then
		-- display pop-up for adding DKP or GR to the character selected.
		RDKP_SettingTableForAdjustment.SettingName = getglobal(this:GetName() .. "_ZoneName"):GetText();
		-- If there's no name in the frame, then don't show the frame.:
		if (RDKP_SettingTableForAdjustment.SettingName) then
			RDKP_SettingDetailName:SetText(RDKP_SettingTableForAdjustment.SettingName);  
			RDKP_SettingDetailGreenText:SetText(RDKP_GR_VALUE_TEMP["ff1eff00"][RDKP_SettingTableForAdjustment.SettingName]);
			RDKP_SettingDetailBlueText:SetText(RDKP_GR_VALUE_TEMP["ff0070dd"][RDKP_SettingTableForAdjustment.SettingName]);
			RDKP_SettingDetailEpicText:SetText(RDKP_GR_VALUE_TEMP["ffa335ee"][RDKP_SettingTableForAdjustment.SettingName]);
			RDKP_SettingTableForAdjustment["ff1eff00"] = RDKP_GR_VALUE_TEMP["ff1eff00"][RDKP_SettingTableForAdjustment.SettingName];
			RDKP_SettingTableForAdjustment["ff0070dd"] = RDKP_GR_VALUE_TEMP["ff0070dd"][RDKP_SettingTableForAdjustment.SettingName];
			RDKP_SettingTableForAdjustment["ffa335ee"] = RDKP_GR_VALUE_TEMP["ffa335ee"][RDKP_SettingTableForAdjustment.SettingName];
			RDKP_SettingDetailFrame:Show();
		end
	end
end

StaticPopupDialogs["SAVE_AUDIT"] = {
	text = format(TEXT("Save Adjustment to DKP and/or GR values?"), "RDKP"),
	button1 = TEXT(YES),
	button2 = TEXT(NO),
	OnAccept = function()
		local szTrans = RDKP_Transaction_Adjustment(RDKP_TableForAudit.PlayerName, RDKP_TableForAudit.DKP, RDKP_TableForAudit.GR);
		if szTrans then 
			local bResults = RDKP_Guild_SendInfo(szTrans);
			if ( not bResults ) then
				-- Problem 
				RDKP_Debug("DKP Not Awarded for " .. szName);
			end
		end
		RDKP_Table_RaidInfo["RDKP_RaidID"] = nil;
		-- Hide the frame.
		RDKP_MemberDetailFrame:Hide();
		RDKP_PopulatePriorityButtons();
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
};

StaticPopupDialogs["SAVE_SETTING"] = {
	-- Writes from the temp table holding 1 setting to the temp table of all settings.
	text = format(TEXT("Save Adjustment to GR values for this Zone?"), "RDKP"),
	button1 = TEXT(YES),
	button2 = TEXT(NO),
	OnAccept = function()
		RDKP_GR_VALUE_TEMP["ff1eff00"][RDKP_SettingTableForAdjustment.SettingName] = RDKP_SettingTableForAdjustment["ff1eff00"];
		RDKP_GR_VALUE_TEMP["ff0070dd"][RDKP_SettingTableForAdjustment.SettingName] = RDKP_SettingTableForAdjustment["ff0070dd"];
		RDKP_GR_VALUE_TEMP["ffa335ee"][RDKP_SettingTableForAdjustment.SettingName] = RDKP_SettingTableForAdjustment["ffa335ee"];
		-- Hide the frame.
		RDKP_SettingDetailFrame:Hide();
		RDKP_PopulateSettingsFrames();
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
};

StaticPopupDialogs["ADJUST_DKP"] = {
	text = TEXT("Adjust DKP: "),
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	hasEditBox = 1,
	maxLetters = 10,
	hasWideEditBox = 1,
	OnAccept = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		RDKP_AdjustDKPText:SetText(editBox:GetText());
		RDKP_TableForAudit["DKP"] = tonumber(editBox:GetText());
	end,
	OnShow = function()
		getglobal(this:GetName().."WideEditBox"):SetText(RDKP_TableForAudit.DKP);
		getglobal(this:GetName().."WideEditBox"):SetFocus();
	end,
	EditBoxOnEnterPressed = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		RDKP_AdjustDKPText:SetText(editBox:GetText());
		RDKP_TableForAudit["DKP"] = tonumber(editBox:GetText());
	end,
	EditBoxOnEscapePressed = function()
		this:GetParent():Hide();
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
};

StaticPopupDialogs["ADJUST_GR"] = {
	text = TEXT("Adjust GR: "),
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	hasEditBox = 1,
	maxLetters = 10,
	hasWideEditBox = 1,
	OnAccept = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		RDKP_AdjustGRText:SetText(editBox:GetText());
		RDKP_TableForAudit.GR = tonumber(editBox:GetText());
	end,
	OnShow = function()
		getglobal(this:GetName().."WideEditBox"):SetText(RDKP_TableForAudit.GR);
		getglobal(this:GetName().."WideEditBox"):SetFocus();
	end,
	EditBoxOnEnterPressed = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		RDKP_AdjustGRText:SetText(editBox:GetText());
		RDKP_TableForAudit.GR = tonumber(editBox:GetText());
	end,
	EditBoxOnEscapePressed = function()
		this:GetParent():Hide();
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
};

StaticPopupDialogs["ADJUST_DKP_SETTING"] = {
	text = TEXT("Adjust DKP Setting: "),
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	hasEditBox = 1,
	maxLetters = 10,
	hasWideEditBox = 1,
	OnAccept = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		RDKP_TABLE_DKP_VALUE_TEMP[RDKP_DKPKEY_table_Map[RDKP_Setting_Selected]] = tonumber(editBox:GetText());
	end,
	OnShow = function()
		getglobal(this:GetName().."WideEditBox"):SetText(RDKP_TABLE_DKP_VALUE_TEMP[RDKP_DKPKEY_table_Map[RDKP_Setting_Selected]]);
		getglobal(this:GetName().."WideEditBox"):SetFocus();
	end,
	EditBoxOnEnterPressed = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		RDKP_TABLE_DKP_VALUE_TEMP[RDKP_DKPKEY_table_Map[RDKP_Setting_Selected]] = tonumber(editBox:GetText());
	end,
	EditBoxOnEscapePressed = function()
		this:GetParent():Hide();
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
};

StaticPopupDialogs["ADJUST_GREEN"] = {
	text = TEXT("Adjust UnCommon: "),
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	hasEditBox = 1,
	maxLetters = 10,
	hasWideEditBox = 1,
	OnAccept = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		RDKP_AdjustGreenText:SetText(editBox:GetText());
		RDKP_SettingTableForAdjustment["ff1eff00"] = tonumber(editBox:GetText());
	end,
	OnShow = function()
		local iAdjust = 0;
		if (RDKP_SettingTableForAdjustment["ff1eff00"]) then
			iAdjust = RDKP_SettingTableForAdjustment["ff1eff00"]
		end
		getglobal(this:GetName().."WideEditBox"):SetText(RDKP_SettingTableForAdjustment["ff1eff00"]);
		getglobal(this:GetName().."WideEditBox"):SetFocus();
	end,
	EditBoxOnEnterPressed = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		RDKP_AdjustGreenText:SetText(editBox:GetText());
		RDKP_SettingTableForAdjustment["ff1eff00"] = tonumber(editBox:GetText());
	end,
	EditBoxOnEscapePressed = function()
		this:GetParent():Hide();
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
};

StaticPopupDialogs["ADJUST_BLUE"] = {
	text = TEXT("Adjust Rare: "),
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	hasEditBox = 1,
	maxLetters = 10,
	hasWideEditBox = 1,
	OnAccept = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		RDKP_AdjustBlueText:SetText(editBox:GetText());
		RDKP_SettingTableForAdjustment["ff0070dd"] = tonumber(editBox:GetText());
	end,
	OnShow = function()
		local iAdjust = 0;
		if (RDKP_SettingTableForAdjustment["ff0070dd"]) then
			iAdjust = RDKP_SettingTableForAdjustment["ff1eff00"]
		end
		getglobal(this:GetName().."WideEditBox"):SetText(RDKP_SettingTableForAdjustment["ff0070dd"]);
		getglobal(this:GetName().."WideEditBox"):SetFocus();
	end,
	EditBoxOnEnterPressed = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		RDKP_AdjustBlueText:SetText(editBox:GetText());
		RDKP_SettingTableForAdjustment["ff0070dd"] = tonumber(editBox:GetText());
	end,
	EditBoxOnEscapePressed = function()
		this:GetParent():Hide();
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
};

StaticPopupDialogs["ADJUST_EPIC"] = {
	text = TEXT("Adjust Epic: "),
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	hasEditBox = 1,
	maxLetters = 10,
	hasWideEditBox = 1,
	OnAccept = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		RDKP_AdjustEpicText:SetText(editBox:GetText());
		RDKP_SettingTableForAdjustment["ffa335ee"] = tonumber(editBox:GetText());
	end,
	OnShow = function()
		local iAdjust = 0;
		if (RDKP_SettingTableForAdjustment["ffa335ee"]) then
			iAdjust = RDKP_SettingTableForAdjustment["ffa335ee"]
		end
		getglobal(this:GetName().."WideEditBox"):SetText(RDKP_SettingTableForAdjustment["ffa335ee"]);
		getglobal(this:GetName().."WideEditBox"):SetFocus();
	end,
	EditBoxOnEnterPressed = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		RDKP_AdjustEpicText:SetText(editBox:GetText());
		RDKP_SettingTableForAdjustment["ffa335ee"] = tonumber(editBox:GetText());
	end,
	EditBoxOnEscapePressed = function()
		this:GetParent():Hide();
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
};

