-----------------------------------------
-- Addon: Relational DKP
-- By: Shazear 
-- Shizukana <Veterans of the Phoenix>, Malygos
-- e-mail: shazear@hotmail.com
-- Code also used from CTRT
-- References: 
-----------------------------------------

-- When the AddOn loads...
function RDKP_OnLoad()
	-- Register the game events neccesary for our functionality
	this:RegisterEvent("VARIABLES_LOADED");	
	-- This list stolen from CTRT 
	-- I'm hoping to hook onto the CTRT information gathering so I don't have to do the work also.
	this:RegisterEvent("CHAT_MSG_LOOT");
	this:RegisterEvent("CHAT_MSG_WHISPER");
	this:RegisterEvent("RAID_ROSTER_UPDATE");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("GUILD_ROSTER_UPDATE");
	-- Additional events that I may need information about:
	this:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH");
	this:RegisterEvent("CHAT_MSG_ADDON");
	-- Register Slash Commands:
	SLASH_RDKP_SHOW1 = "/rdkp";
	SlashCmdList["RDKP_SHOW"] = RDKP_ShowUI;
	SLASH_RDKP_RESET1 = "/rdkp_resetall";
	SlashCmdList["RDKP_RESET"] = RDKP_ResetData;
end

-- Based upon our registered events, we want different things to happen when each event occurs.
function RDKP_OnEvent(event)
	-- We will have to do our own work during the events, but I believe we can leverage the CTRT data.
	-- The question is will CTRT's OnEvent fire before us or after?
	-- Looks like CTRT processes events happen after us.  :(
	if (event == "VARIABLES_LOADED") then
		RDKP_Debug("VARIABLES_LOADED");
		-- Saved Variables should now be ready.
		-- Any last minute initialization should happen here.
		-- Update Total Player List.
		RDKP_Print("Loaded.");
	elseif (event == "PLAYER_ENTERING_WORLD") then
		RDKP_Debug("PLAYER_ENTERING_WORLD");
		-- Should now have player information
		-- Fired when the player enters the world, enters/leaves an instance, or respawns at a graveyard. Also fires any other time the player sees a loading screen.
		GuildRoster();
		RDKP_UpdateMasterPlayerList();
		RDKP_BroadcastRaids();
		RDKP_PlayerMap_Update();
	elseif (event == "CHAT_MSG_LOOT" ) then
		RDKP_Debug("CHAT_MSG_LOOT");
		-- Are we currently in a raid?
		if (RDKP_Table_RaidInfo["RDKP_bCurrentRaid"]) then
			-- When looting, we're going to need to update a given player's GR value.
			-- Need to parse the looting message (code stolen from CTRT)
			RDKP_AwardGRForLoot(arg1);
		end
--	elseif (event == "CHAT_MSG_SYSTEM") then
--		RDKP_Debug("CHAT_MSG_SYSTEM");
--		-- Join or Leaves group can happen here.
	elseif (event == "CHAT_MSG_WHISPER") then
		RDKP_Debug("CHAT_MSG_WHISPER");
		RDKP_Debug("From: " .. arg2);
		-- for hooking tells for filtering looting process. 
		if (RDKP_Table_RaidInfo.RDKP_bCurrentRaid and RDKP_FilterBy.PST) then
			-- Receive who sent the message.
			-- if they're in the raid, add it to the RDKP_RaidTellList
			if (not RDKP_RaidTellList) then
				RDKP_RaidTellList = {}
			end
			if (RDKP_SearchList(RDKP_Table_RaidPlayers, arg2)) then
				tinsert(RDKP_RaidTellList, arg2);
				-- And refresh the Priority List.
				RDKP_PopulatePriorityButtons();
			end
			-- Tells not from the Raid, will be be ignored for this purpose.
		end
		-- I'm seeing wierd extra whisper events from various characters while running around.
		-- This could cause in-advertant additions to the list.
		-- Looks like GuildAds is causing the extra whisper traffic.
	elseif (event == "GUILD_ROSTER_UPDATE") then
		-- Hoping that people adding/leaving/guildie log-in/etc.  
		-- Need to experiment.
		RDKP_Debug("GUILD_ROSTER_UPDATE");
		-- If this does what I want, we may need to update MasterPlayerList() here.
		RDKP_UpdateMasterPlayerList();
		RDKP_PlayerMap_Update();
	elseif (event == "RAID_ROSTER_UPDATE") then
		RDKP_Debug("RAID_ROSTER_UPDATE");
		--Fired whenever a raid is formed or disbanded, players are leaving or joining a raid (unsure if rejected join requests also fire the event), or when looting rules are changed (regardless of being in raid or party!)
		--* As of 1.12.0, this event no longer fires when entering the world as a raid member. It did previously. Bug? --Mikk 08:54, 28 August 2006 (EDT) 
		RDKP_UpdateRaidList();
		-- Determine if we're in a raid or creating one:
		if GetNumRaidMembers() == 0 then
			-- We're leaving raid.
			-- So we should shut down the raid logging here.
			RDKP_EndRaid();
			return
		end 
		if (RDKP_Table_RaidInfo["RDKP_bCurrentRaid"]) then
			-- We already are in a raid.
			-- So rules have changed, or roster has changed.
			RDKP_Raid_BroadcastKey(RDKP_Table_RaidInfo["RDKP_RaidID"]);
		end
	elseif (event == "CHAT_MSG_COMBAT_HOSTILE_DEATH") then
		RDKP_Debug("CHAT_MSG_COMBAT_HOSTILE_DEATH");
		-- This is where per-kill DKP is handled.
		-- arg1 is the string that contains the information we want. 
		-- Get Mob Killed string from arg1
		if ((RDKP_Table_RaidInfo["RDKP_bCurrentRaid"]) and (RDKP_IsDKPOfficer())) then
			local iStart, iEnd, szMobDied = string.find(arg1, "(.+) dies.");
			if ( szMobDied == nil ) then
				local iStart, iEnd, szMobDied = string.find(arg1, "You have slain (.+)!");
			end
			RDKP_Debug(szMobDied);
			-- Check string to pre-defined list of bosses
			b_Boss_Killed = nil;
			for k, szboss in RDKP_Table_Bosses do
				-- Really need to make sure this code works.  
				if ( szMobDied == szboss) then
					-- if it's on this list, then award DKP to raid.
					b_Boss_Killed = true;
				end
			end
			-- If the boss is the correct boss to kill or
			-- If we're in debug mode, so all kills award DKP
			RDKP_ConditionalDataReset();
			if ( b_Boss_Killed ) then
				RDKP_AwardDKPForKill("kill");
			elseif ((RDKP_TABLE_DKP_VALUE) and (RDKP_TABLE_DKP_VALUE["trash"])) then
				RDKP_AwardDKPForKill("trash");
			end
		end
	elseif (event == "CHAT_MSG_ADDON") then
		-- Need to see what else comes w/ this.  I can expect 3 arguments?
		-- Add-ons can send hidden chat messages to players in the party, raid or guild, using SendAddonMessage("prefix", "message", "PARTY"|"RAID"|"GUILD"|"BATTLEGROUND"). 
		-- When players receive these messages, a new event "CHAT_MSG_ADDON" is sent, with arg1-arg3 being the parameters to SendAddonMessage(), and arg4 
		-- being the name of the player that sent it. The combined length of "prefix" and "message" must be less than or equal to 254 bytes.
		if (arg1 == RDKP_SZ_ADDON_PREFIX) then
--			RDKP_Debug("CHAT_MSG_ADDON: Ours.");
			-- Get the information from the given transmition
			RDKP_RecieveInfo(arg2, arg3, arg4);
		elseif (arg1 == RDKP_SZ_ADDON_PREFIX_OLD) then
			-- TODO: need to change the evaluation to a table of old values.
			-- Previous interface versions.
			RDKP_Debug("WARNING: " .. arg4 .. " is using an old Version.  They need to update their RDKP.");
		else
			-- Let's see who else is doing the chatting.
			-- RDKP_Debug("CHAT_MSG_ADDON: " .. arg1);
		end
	end
end

function RDKP_Print(...)
	-- Code taken from CTRT's Debug output code w/ only slight modification.
	local sDebug = "Relational DKP: ";
	for i = 1, arg.n, 1 do
		if ( arg[i] ) then
			sDebug = sDebug .. arg[i];
		end
	end
	-- MessageFrame:AddMessage("text", r, g, b, a, holdTime) 
	DEFAULT_CHAT_FRAME:AddMessage(sDebug, 1, .25, .75);
end

function RDKP_Debug(...)
	-- Code taken from CTRT's Debug output code w/ only slight modification.
	if ( RDKP_b_Debug ) then
		local sDebug = "# RDKP_DBG: ";
		for i = 1, arg.n, 1 do
			if ( arg[i] ) then
				sDebug = sDebug .. arg[i] .. "#";
			end
		end
		-- MessageFrame:AddMessage("text", r, g, b, a, holdTime) 
		DEFAULT_CHAT_FRAME:AddMessage(sDebug, 1, 0.5, 0);
	end
end

-----------------------------------------------------------
-- Data Functions
-- 
--
function RDKP_ResetData()
	RDKP_Print("All data being wiped, reset to system defaults, and repopulated as best as possible.");
	-- Wipe out existing data
	RDKP_Table_CharacterMap =  nil;
	RDKP_Table_CharacterMap = {[RDKP_SZ_MYCHARACTERS] = nil,}
--	RDKP_Table_Raid_Register = nil;
--	RDKP_Table_Raid_Register = {}
	RDKP_Table_MasterPlayers = nil;
	RDKP_Table_MasterPlayers = {}
	RDKP_Table_PlayersSorted = nil;
	RDKP_Table_PlayersSorted = {}
	RDKP_TABLE_DKP_VALUE = RDKP_TABLE_DKP_VALUE_SYSTEMDEFAULT;
	RDKP_GR_VALUE = RDKP_GR_VALUE_SYSTEMDEFAULT;
	-- Rebuild from scratch:
	RDKP_PlayerMap_Update();
	RDKP_UpdateMasterPlayerList();
	RDKP_BroadcastRaids();
	RDKP_ReparseTransactions();
	RDKP_SortMasterListbyPriority();
	RDKP_SendSettingsUpdateRequest();
end

function RDKP_ResetRegister()
	RDKP_Table_Raid_Register = nil;
	RDKP_Table_Raid_Register = {}
end

function RDKP_ConditionalDataReset()
	if (not RDKP_GR_VALUE) then
		RDKP_Print("WARNING: Gear Values were nil, restoring from system defaults.  Resetting Timestamp, and requesting any settings updates.");
		RDKP_GR_VALUE = RDKP_GR_VALUE_SYSTEMDEFAULT;
	end
	if (not RDKP_TABLE_DKP_VALUE) then
		RDKP_Print("WARNING: DKP Values were nil, restoring from system defaults.  Resetting Timestamp, and requesting any settings updates.");
		RDKP_TABLE_DKP_VALUE = RDKP_TABLE_DKP_VALUE_SYSTEMDEFAULT;
	end
	RDKP_SendSettingsUpdateRequest();
end

function RDKP_SearchList(table, sz)
	RDKP_Debug("RDKP_SearchList()")
	if sz then 
		RDKP_Debug("sz: " .. sz)
	end
	if (table and sz) then
		for k, v in table do
			if (string.lower(v) == string.lower(sz)) then
				return true;
			end
		end
	end
	return nil;
end

function RDKP_UpdateRaidList()
	RDKP_Debug("RDKP_UpdateRaidList()");
	RDKP_Table_RaidPlayers = {};
	for iRaidMember = 1, GetNumRaidMembers(), 1 do 
		local szPlayerName, rank, subgroup, level, class, fileName, szZone, online, isDead = GetRaidRosterInfo(iRaidMember);
		tinsert(RDKP_Table_RaidPlayers, szPlayerName);
	end
end
function RDKP_Build_RaidKeyID()
	-- We need a way to have a consistent unique ID for each raid.
	-- This function should be used by the RL at the time the raid start is triggered.
	-- What will be unique to a given raid?
	-- RaidLeader & Date/Time at time of creation.
	-- We need to call this only when the raid is created.
	RDKP_Debug("RDKP_Build_RaidKeyID()");
	local szName = UnitName("player");
	RDKP_Table_RaidInfo["RDKP_RaidID"] = szName .. CT_RaidTracker_Date(); -- NOTE: This is pulling the local system date, not the server date.
	RDKP_Debug("RaidKeyID = " .. RDKP_Table_RaidInfo["RDKP_RaidID"]);
	return RDKP_Table_RaidInfo["RDKP_RaidID"];
end

function RDKP_IsDKPOfficer()
	-- Return true if current officer has the rights to grant somone additional DKP/GR.
	-- For right now, this is based purely off the ability to edit the officer's note.
	-- At some point, these should be able to be set in the options.
	return CanEditOfficerNote();
end

function RDKP_ReparseTransactions()
	-- There may be times when the current MasterPlayerList isn't up-to-date w/ all the data available in local transactions.
	RDKP_Debug("RDKP_ReparseTransactions()");
	-- First set all players DKP & GR values to 0
	for players,playerData in RDKP_Table_MasterPlayers do
		playerData.DKP = 0;
		playerData.GR = 1;
		playerData.PRI = 0;
	end
	-- Iterate through the RaidRegister and reparse that information.
	for raidID,raidTransactions in RDKP_Table_Raid_Register do
		for TransactionID, TransactionValue in raidTransactions do
			RDKP_UpdateSummarywithTransaction(TransactionValue["player"], TransactionValue["DKP"], TransactionValue["GR"]);
		end
	end
end

function RDKP_UpdateMasterPlayerList()
	local iLenGuildList = GetNumGuildMembers(true);
	for i = 0, iLenGuildList, 1 do
		local szName, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
		if (szName) then 
--			RDKP_Debug("RDKP_UpdateMasterPlayerList(): " .. szName);
			if (not RDKP_Table_MasterPlayers) then
				-- no Master table - need to create it.
				RDKP_Table_MasterPlayers[szName] = {
					["NAME"] = szName, 
					["DKP"] = 0, 
					["GR"] = 1,
					["PRI"] = 0
				}
			elseif (not (RDKP_Table_MasterPlayers[szName])) then
				-- Build the info on the table
--				tinsert(RDKP_Table_MasterPlayers, szName);
				RDKP_Table_MasterPlayers[szName] = {
					["NAME"] = szName, 
					["DKP"] = 0, 
					["GR"] = 1,
					["PRI"] = 0
				}
			else
				-- Found on the master list of players
				-- Re-calc priority just incase it's not accurate.
				RDKP_Table_MasterPlayers[szName]["PRI"] =  RDKP_GetPriority(szName);
			end
		end
	end
	RDKP_SortMasterListbyPriority();
end

function RDKP_StartRaid()
	RDKP_Debug("RDKP_StartingRaid()" );
	-- This is what will be called when we start a raid.
	RDKP_Table_RaidInfo["RDKP_bCurrentRaid"] = true;
	-- Are we the raid leader?
	if (IsRaidLeader()) then
		-- If so create the raid key.
		RDKP_Table_RaidInfo["RDKP_RaidID"] = RDKP_Build_RaidKeyID();
		-- Start awarding DKP based on time.
		RDKP_ConditionalDataReset();
		local iWhen = 3600/RDKP_TABLE_DKP_VALUE["hour"];
		Chronos.scheduleRepeating(RDKP_SZ_DKPFORTIME, iWhen , RDKP_AwardDKPForTime);
	end
end

function RDKP_EndRaid()
	RDKP_Debug("RDKP_EndRaid()" );
	-- This is what will be called when we start a raid.
	RDKP_Table_RaidInfo["RDKP_bCurrentRaid"] = nil;
	-- Are we the raid leader?
	-- This query won't work because if I leave the group, this will return fail.
--	if IsRaidLeader() then
		-- So destroy the local copy of the raid key.
		-- If I rejoin an on-going raid, I will get a key update.
		RDKP_Table_RaidInfo["RDKP_RaidID"] = nil;
		-- Stop awarding DKP based on time.
		Chronos.unscheduleRepeating(RDKP_SZ_DKPFORTIME);
--	end
end

function RDKP_GetPriority(szName)
	if (szName) then
		local priority = RDKP_Table_MasterPlayers[szName]["DKP"]/RDKP_Table_MasterPlayers[szName]["GR"];
		return priority;
	else
		RDKP_Debug("RDKP_GetPriority(): nil")
	end		
end

function RDKP_SortMasterListbyPriority()
	RDKP_Debug("RDKP_SortMasterListbyPriority()");
	if (RDKP_Table_MasterPlayers ~= nil) then
		RDKP_Table_PlayersSorted = RDKP_SortTableByField(RDKP_Table_MasterPlayers, "PRI");
	end
end

function RDKP_SortTableByField(tbl, field)
	local new_tbl = {}
	for k, v in tbl do
		tinsert(new_tbl, k);
	end
	table.sort(new_tbl);
	RDKP_Debug("Sorted" .. field .. "List:");
	table.sort(new_tbl,
		function(a1, a2)
			if (tbl[a1][field] ==  tbl[a2][field]) then
				return (a1 < a2);
			else
				return (tbl[a1][field] > tbl[a2][field]);
			end
		end
	)
	return new_tbl;
end

function RDKP_Debug_OutputMasterList()
	for i, v in RDKP_Table_MasterPlayers do
		RDKP_Debug(i);
		local szCharStats = v.NAME .. ": " .. v["DKP"] .. " / " .. v["GR"] .. " = " .. v["PRI"];
		RDKP_Debug(szCharStats);
	end
end

function RDKP_GetSettings()
	-- Function for gettings all the settings values 
	local szDKPSettings = "DKP Settings: "; 
	RDKP_ConditionalDataReset();
	for k,v in RDKP_TABLE_DKP_VALUE do
		szDKPSettings = szDKPSettings .. "\n" .. k .. " = " .. v;
	end
	local szGRSettings = "Gear Values: ";
	for k,v in RDKP_GR_VALUE do
		szGRSettings = szGRSettings .. "\n" .. k .. ": ";
		for i, j in v do 
			szGRSettings = szGRSettings .. "\n" .. i .. " = " .. j;
		end 
	end
	return szDKPSettings, szGRSettings;
end

function RDKP_BroadcastRaids()
	RDKP_Debug("RDKP_BroadcastRaids()" );
	-- Take the list of all the raids I know about and broadcast the RaidIDs
	-- We probably want to send the size of the raid as well, so that we can determine if the raids we do have are partials.
	if (RDKP_Table_Raid_Register) then
		for raid, raidvalue in RDKP_Table_Raid_Register do
			-- Don't want to broadcast the current raid because it's still in-flux.
			if (RDKP_Table_RaidInfo["RDKP_RaidID"]) then 
				if (raid ~= RDKP_Table_RaidInfo["RDKP_RaidID"]) then
					local ilength = table.getn(raidvalue);
					RDKP_Guild_SendInfo(RDKP_SZ_HAVERAID_PREFIX .. raid .. RDKP_SZ_FIELD_DELIMITER .. ilength);
				end
			else
				local ilength = table.getn(raidvalue);
				RDKP_Guild_SendInfo(RDKP_SZ_HAVERAID_PREFIX .. raid .. RDKP_SZ_FIELD_DELIMITER .. ilength);
			end
		end
	end
end

function RDKP_BroadcastRaidTransactions(szRaidID)
	RDKP_Debug("RDKP_BroadcastRaidTransactions()" .. szRaidID);
	if RDKP_Table_Raid_Register[szRaidID] then
		for trans, v in RDKP_Table_Raid_Register[szRaidID] do
		-- Package up the transactions just like we did before if the raid was going on now.
			RDKP_Guild_SendInfo(RDKP_Build_Transaction(szRaidID, trans, v["player"], v["DKP"], v["GR"]));
		end
	end
end

-----------------------------------------------------------
-- Player Data Functions
-- 
--
function RDKP_PlayerMap_Update()
	-- Update the list of the current characters for the player.
	RDKP_PlayerMap_AddMe();
	-- Now that we have a list, we want each of the characters on the list
	-- to point to the main character.
	-- This allows me to look at the character map, and use any character to assign points
	-- to the main already provided.
	for k, v in RDKP_Table_CharacterMap[RDKP_SZ_MYCHARACTERS] do
		RDKP_Table_CharacterMap[v] = { 
			RDKP_SZ_MAINCHAR = RDKP_Table_CharacterMap[RDKP_SZ_MYMAINCHAR],
			RDKP_SZ_LASTUPDATE  = date(); -- Not using pretty date here because we need it for comparison purposes.  Need to check to see if I can get this to be a straight number.
		}
	end
	-- This takes care of the current character, however there's no way ATM to communicate this to everyone else.
	RDKP_PlayerMap_Broadcast();
end

function RDKP_PlayerMap_Broadcast()
	-- Publish all the available characters on the PlayerMap.
	RDKP_Debug("RDKP_PlayerMap_Broadcast()");
	for k, v in RDKP_Table_CharacterMap do
		if (k ~= RDKP_SZ_MYCHARACTERS) and (k ~= RDKP_SZ_MYMAINCHAR) then
			local szInfo = RDKP_SZ_CHAR_TRANSMIT_PREFIX .. k .. RDKP_SZ_FIELD_DELIMITER .. v["RDKP_SZ_MAINCHAR"] .. RDKP_SZ_FIELD_DELIMITER .. v["RDKP_SZ_LASTUPDATE"];
			RDKP_Guild_SendInfo(szInfo);
		end
	end
end

function RDKP_RecievePlayerMapEntry(szInfo)
	local pattern = "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)"
	local iStart, iEnd, szEntry, szMain, iUpdate = string.find(szInfo, pattern);
	if (RDKP_Table_CharacterMap[szEntry] == nil) then
		RDKP_Table_CharacterMap[szEntry] = {
			RDKP_SZ_MAINCHAR = szMain,
			RDKP_SZ_LASTUPDATE  = iUpdate; -- Not using pretty date here because we need it for comparison purposes.
		}
	elseif (RDKP_Table_CharacterMap[szEntry]["RDKP_SZ_LASTUPDATE"] < iUpdate) then
		RDKP_Table_CharacterMap[szEntry] = {
			RDKP_SZ_MAINCHAR = szMain,
			RDKP_SZ_LASTUPDATE  = iUpdate; -- Not using pretty date here because we need it for comparison purposes.
		}
	end
end

function RDKP_PlayerMap_AddMe()
	-- Setup a mapping for all characters associated w/ this account.
	-- First, get the name of the current character
	local szPlayer = UnitName("player");
	-- Check to see if there's a "MyCharacters" entry - 
	if (RDKP_Table_CharacterMap) then 
		if (RDKP_Table_CharacterMap[RDKP_SZ_MYCHARACTERS] == nil) then
			-- If not, create it, w/ the initial value of the current character.
			RDKP_Table_CharacterMap[RDKP_SZ_MYCHARACTERS] = {szPlayer};
			-- Also set the current character as the main for this account.
			RDKP_PlayerMap_SetAsMain();
		else
			-- If so, add the current character to the list.
			-- Check to see if the character is already there:
			local bOnList = nil;
			for k, v in RDKP_Table_CharacterMap[RDKP_SZ_MYCHARACTERS] do
				if ( v == szPlayer) then 
					bOnList = true;
				end
			end
			if (not bOnList) then
				-- if not add him.
				tinsert(RDKP_Table_CharacterMap[RDKP_SZ_MYCHARACTERS], szPlayer);
			end
		end
	else 
		RDKP_Table_CharacterMap = {
			[RDKP_SZ_MYCHARACTERS] = {szPlayer}
		};
		RDKP_PlayerMap_SetAsMain();
	end
end

function RDKP_PlayerMap_AddNotMe(szName)
	-- Setup a mapping for all characters associated w/ this account.
	-- First, get the name of the current character
	RDKP_Table_CharacterMap[szName] = {
		RDKP_SZ_LASTUPDATE = date(),
		RDKP_SZ_MAINCHAR = szName,
	}
end

function RDKP_PlayerMap_SetAsMain()
	-- Set this character as the main, to track all settings.
	-- May have to have a MyMain by guild.
	-- But for now, we're going to leave it like this.
	RDKP_Table_CharacterMap[RDKP_SZ_MYMAINCHAR] = UnitName("player")
	-- Update the guild w/ this information:
	RDKP_PlayerMap_Update();
end

function RDKP_GetPlayers()
	RDKP_Debug("RDKP_GetPlayers()");
	local szCharStats = "";
	for k, v in RDKP_Table_MasterPlayers do
		szCharStats = szCharStats .. "\n" .. k .. ": " .. v["DKP"] .. " / " .. v["GR"] .. " = " .. v["PRI"];
	end
	return szCharStats;
end 

-----------------------------------------------------------
-- Settings Functions
-- At some point we will want all clients to be able to have this information.
--
function RDKP_SendSettingsUpdateRequest()
	local szDKPDate = RDKP_TABLE_DKP_VALUE.date;
	local szGRDate = RDKP_GR_VALUE.date;
	local szSettingsRequest = RDKP_SZ_SETTINGSREQUEST .. RDKP_SZ_FIELD_DELIMITER;
	if (szDKPDate) then
		szSettingsRequest = szSettingsRequest .. szDKPDate;
	end 
	szSettingsRequest = szSettingsRequest .. RDKP_SZ_FIELD_DELIMITER;
	if (szGRDate) then
		szSettingsRequest = szSettingsRequest .. szGRDate;
	end
	szSettingsRequest = szSettingsRequest .. RDKP_SZ_FIELD_DELIMITER;
	RDKP_Guild_SendInfo(szSettingsRequest);
end

function RDKP_ReceiveSettingsUpdateRequest(szInfo)
	-- Recieve the request for new settings
	-- Matches the pattern:
	local szPattern = RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER
	local iStart, iEnd, dateDKP, dateGR = string.find(szInfo, szPattern)
	if (not dateDKP) then 
		if (RDKP_TABLE_DKP_VALUE.date) then
			local bDKP = 1;
		end
	elseif (dateDKP) then
		if (not RDKP_TABLE_DKP_VALUE.date) or (dateDKP < RDKP_TABLE_DKP_VALUE.date) then
			local bDKP = 1;
		end
	end
	if (not dateGR) then
		if (RDKP_GR_VALUE.date) then
			local bGR = 1;
		end
	elseif (dateGR) then
		if (RDKP_GR_VALUE.date) or (dateGR < RDKP_GR_VALUE.date) then
			local bGR = 1;
		end
	end
	RDKP_SendSettingsUpdate(bDKP, bGR);
end

function RDKP_SendSettingsUpdate(bDKP, bGR)
	-- By the time this is called, we should already know if we have newer info
	-- than the requester.
	if (bDKP) then
		local szSettingsUpdateDKP = RDKP_SZ_SETTINGSUPDATE_DKP .. RDKP_SZ_FIELD_DELIMITER .. RDKP_TABLE_DKP_VALUE.date .. RDKP_SZ_FIELD_DELIMITER .. RDKP_TABLE_DKP_VALUE.trash .. RDKP_SZ_FIELD_DELIMITER .. RDKP_TABLE_DKP_VALUE.kill .. RDKP_SZ_FIELD_DELIMITER .. RDKP_TABLE_DKP_VALUE.hour .. RDKP_SZ_FIELD_DELIMITER .. RDKP_TABLE_DKP_VALUE.raid_start .. RDKP_SZ_FIELD_DELIMITER;
		RDKP_Guild_SendInfo(szSettingsUpdateDKP);
		szSettingsUpdateDKP = nil;
	end
	if (bGR) then
		-- let's start w/ getting our header built.  
		-- This may have to broken up into multiple packets.
		local szSettingsUpdateHeader = RDKP_SZ_SETTINGSUPDATE_GR .. RDKP_SZ_FIELD_DELIMITER .. RDKP_GR_VALUE.date .. RDKP_SZ_FIELD_DELIMITER;
		-- First, Greens:
		local szSettingsUpdateGreen = szSettingsUpdateHeader .. "ff1eff00" .. RDKP_SZ_FIELD_DELIMITER ;
		for k,v in RDKP_GR_VALUE.ff1eff00 do
			szSettingsUpdateGreen = szSettingsUpdateGreen .. RDKP_Table_Zone_Map[k] .. RDKP_SZ_FIELD_DELIMITER .. v .. RDKP_SZ_FIELD_DELIMITER;
		end
		local results = RDKP_Guild_SendInfo(szSettingsUpdateGreen);
		szSettingsUpdateGreen = nil;
		-- do something w/ the results.
		if (not results) then
			RDKP_Print("Error when posting settings update in Uncommons.")
		end
		-- Next, blues:
		local szSettingsUpdateBlue = szSettingsUpdateHeader .. "ff0070dd" .. RDKP_SZ_FIELD_DELIMITER ;
		for k,v in RDKP_GR_VALUE.ff0070dd do
			szSettingsUpdateBlue = szSettingsUpdateBlue .. RDKP_Table_Zone_Map[k] .. RDKP_SZ_FIELD_DELIMITER .. v .. RDKP_SZ_FIELD_DELIMITER;
		end
		local results = RDKP_Guild_SendInfo(szSettingsUpdateBlue);
		szSettingsUpdateBlue = nil;
		if (not results) then
			RDKP_Print("Error when posting settings update in Rares.")
		end
		-- Finally, epics:
		local szSettingsUpdateEpic = szSettingsUpdateHeader .. "ffa335ee" .. RDKP_SZ_FIELD_DELIMITER ;
		for k,v in RDKP_GR_VALUE.ffa335ee do
			szSettingsUpdateEpic = szSettingsUpdateEpic .. RDKP_Table_Zone_Map[k] .. RDKP_SZ_FIELD_DELIMITER .. v .. RDKP_SZ_FIELD_DELIMITER;
		end
		local results = RDKP_Guild_SendInfo(szSettingsUpdateEpic);
		szSettingsUpdateEpic = nil;
		if (not results) then
			RDKP_Print("Error when posting settings update in Epics.")
		end
	end
end

function RDKP_ReceiveSettingsUpdateDKP(szInfo)
	-- Receiving DKP Info:  
	-- Matching this setup: 
	-- RDKP_SZ_FIELD_DELIMITER .. RDKP_TABLE_DKP_VALUE.date .. RDKP_SZ_FIELD_DELIMITER .. RDKP_TABLE_DKP_VALUE.trash .. RDKP_SZ_FIELD_DELIMITER .. RDKP_TABLE_DKP_VALUE.kill .. RDKP_SZ_FIELD_DELIMITER .. RDKP_TABLE_DKP_VALUE.hour .. RDKP_SZ_FIELD_DELIMITER .. RDKP_TABLE_DKP_VALUE.raid_start .. RDKP_SZ_FIELD_DELIMITER;
	local szPattern = RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)";
	local iStart, iEnd, date, iTrash, iKill, iHour, iRaid, szRemainder = string.find(szInfo, szPattern)
	if (iStart) then -- First check if we even have a match.
		if (date) then
			if ((not RDKP_TABLE_DKP_VALUE.date) or (date > RDKP_TABLE_DKP_VALUE.date)) then
				RDKP_TABLE_DKP_VALUE.date = date;
				if iTrash then
					RDKP_TABLE_DKP_VALUE.trash = tonumber(iTrash);
				end
				if iKill then
					RDKP_TABLE_DKP_VALUE.kill = tonumber(iKill);
				end
				if iHour then
					RDKP_TABLE_DKP_VALUE.hour = tonumber(iHour);
				end 
				if iRaid then
					RDKP_TABLE_DKP_VALUE.raid_start = tonumber(iRaid);
				end
			end
		end
	else
		RDKP_Debug("DKP: NoMatch in " .. szInfo);
	end
end

function RDKP_ReceiveSettingsUpdateGR(szInfo)
	-- This is much more complex of an information gathering problem.
	RDKP_Debug("RDKP_ReceiveSettingsUpdateGR: " .. szInfo);
	local szPattern = RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)";
	local iStart, iEnd, date, szQuality, szData = string.find(szInfo, szPattern);
	if (iStart) then -- First check if we even have a match.
		if (date) then
			if ((not RDKP_GR_VALUE.date) or (date > RDKP_GR_VALUE.date)) then
				RDKP_GR_VALUE["date"] = date;
				while string.len(szData) > 4 do
					RDKP_Debug("UpdateGRwhileLoop: " .. szData);
					local szPattern = "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)"
					local iStart, iEnd, k, v, szData = string.find(szData, szPattern);
					RDKP_GR_VALUE[szQuality][RDKP_Table_Zone_Map[k]] = v;
				end
			end
		end
	else
		RDKP_Debug("GR: NoMatch in " .. szInfo);
	end
end

function RDKP_SaveSettings()
	-- Set the values from the holding area, to the final saved area.
	RDKP_GR_VALUE = RDKP_GR_VALUE_TEMP;
	RDKP_GR_VALUE.date = date();
	RDKP_TABLE_DKP_VALUE = RDKP_TABLE_DKP_VALUE_TEMP;
	RDKP_TABLE_DKP_VALUE.date = date();
	-- Finally, transmit settings changes to the guild.
	RDKP_SendSettingsUpdate(1, 1);
end

-----------------------------------------------------------
-- Communication Functions
-- At some point we will want all clients to be able to have this information.
--
function RDKP_Raid_BroadcastKey(RID)
	RDKP_Debug("RDKP_Raid_BroadcastKey(" .. RID .. ")" );
	-- Only the RaidLeader is allowed to post the key.
	if ( IsRaidLeader() ) then
		if (RID == nil) then
			RDKP_Build_RaidKeyID();
		end
		local RKID = RDKP_SZ_RAIDKEY_PREFIX .. RDKP_Table_RaidInfo["RDKP_RaidID"];
		local bResults = RDKP_Raid_SendInfo(RKID);
		if ( bResults == nil ) then
			RDKP_Debug("Problem on Broadcast." );
		end
	end
end

function RDKP_Raid_SendInfo(szInfo)
	if (szInfo) then 
		RDKP_Debug("RDKP_Raid_SendInfo(".. szInfo .. ")" );
	else
		RDKP_Debug("RDKP_Raid_SendInfo(nil)" );
		return nil;
	end
	-- SendAddonMessage("prefix", "text", "PARTY|RAID|GUILD|BATTLEGROUND")
	-- Max len for prefix + text = 254
	if (string.len(RDKP_SZ_ADDON_PREFIX .. szInfo) < 254) then
		SendAddonMessage(RDKP_SZ_ADDON_PREFIX, szInfo, "RAID");
		return 1;
	else
		RDKP_Debug("String Too Long for transmission.");
		return nil;
	end
end

function RDKP_Guild_SendInfo(szInfo)
	if (szInfo) then 
		RDKP_Debug("RDKP_Guild_SendInfo(".. szInfo .. ")" );
	else
		RDKP_Debug("RDKP_Guild_SendInfo(nil)" );
		return nil;
	end
	-- SendAddonMessage("prefix", "text", "PARTY|RAID|GUILD|BATTLEGROUND")
	-- Max len for prefix + text = 254
	if (string.len(RDKP_SZ_ADDON_PREFIX .. szInfo) < 254) then
		SendAddonMessage(RDKP_SZ_ADDON_PREFIX, szInfo, "GUILD");
		return 1;
	else
		RDKP_Debug("String Too Long for transmission.");
		return nil;
	end
end

function RDKP_RecieveInfo(szInfo, szType, szSource)
	RDKP_Debug("RDKP_RecieveInfo(" .. szInfo .. ", " .. szType .. ", " .. szSource .. ")" );
	-- szInfo should be the same as the szInfo that's sent.
	-- Need to parse out the different pieces of information we could receive.
	if (szType == "RAID") then
		-- Find if a given pattern is in the string.
		-- If so, then return the data bits of that string.
		-- Check if we're getting a new RaidKeyID:
 	 	if ( string.find(szInfo, RDKP_SZ_RAIDKEY_PREFIX) ) then
			local pattern = RDKP_SZ_RAIDKEY_PREFIX .. "(.+)";
			local iStart, iEnd, szCapture = string.find(szInfo, pattern);
			if ( not IsRaidLeader() ) then
				RDKP_Table_RaidInfo["RDKP_RaidID"] = szCapture;
				RDKP_Table_RaidInfo["RDKP_bCurrentRaid"] = 1;
			end
			return true;
		-- Assuming we already have a RaidID, find that related info.
		elseif (string.find(szInfo, RDKP_Table_RaidInfo["RDKP_RaidID"])) then
			-- The data received at this point should match the raid we are presently participating in.
			-- However, check for the Raid ID just incase.
			-- Process a recieved transaction.
			RDKP_Process_Transaction_Raid(szInfo);
			return true;
		end
	elseif (szType == "GUILD") then
		-- For Guild-wide sync:
		-- Settings Request:
		if string.find(szInfo, RDKP_SZ_SETTINGSREQUEST) then
			local pattern = RDKP_SZ_SETTINGSREQUEST .. "(.+)";
			local iStart, iEnd, szCapture = string.find(szInfo, pattern);
			RDKP_ReceiveSettingsUpdateRequest(szCapture);
			return true;
		-- Settings Update DKP:
		elseif string.find(szInfo, RDKP_SZ_SETTINGSUPDATE_DKP) then
			local pattern = RDKP_SZ_SETTINGSUPDATE_DKP .. "(.+)";
			local iStart, iEnd, szCapture = string.find(szInfo, pattern);
			RDKP_ReceiveSettingsUpdateDKP(szCapture);
			return true;
		-- Settings Update GR:
		elseif string.find(szInfo, RDKP_SZ_SETTINGSUPDATE_GR) then
			local pattern = RDKP_SZ_SETTINGSUPDATE_GR .. "(.+)";
			local iStart, iEnd, szCapture = string.find(szInfo, pattern);
			RDKP_ReceiveSettingsUpdateGR(szCapture);
			return true;
		-- Character Map Broadcast:
		elseif string.find(szInfo, RDKP_SZ_CHAR_TRANSMIT_PREFIX) then
			local pattern = RDKP_SZ_CHAR_TRANSMIT_PREFIX .. "(.+)";
			local iStart, iEnd, szCapture = string.find(szInfo, pattern);
			RDKP_RecievePlayerMapEntry(szCapture);
			return true;
		-- RaidList Broadcast
		elseif string.find(szInfo, RDKP_SZ_HAVERAID_PREFIX) then
			local pattern = RDKP_SZ_HAVERAID_PREFIX .. "(.+)";
			local iStart, iEnd, szCapture = string.find(szInfo, pattern);
			RDKP_RecieveRaidID(szCapture);
			return true;
		-- RaidInfo Request.
		elseif string.find(szInfo, RDKP_SZ_REQUESTRAID_PREFIX) then
			local pattern = RDKP_SZ_REQUESTRAID_PREFIX .. "(.+)";
			local iStart, iEnd, szCapture = string.find(szInfo, pattern);
			RDKP_BroadcastRaidTransactions(szCapture);
			return true;
		-- RaidTransactions 
		elseif string.find(szInfo, RDKP_SZ_TRANSACTION_PREFIX) then
			-- The data received at this point should match a historic raid that someone was part of.
			-- Process a recieved transaction.
			local pattern = RDKP_SZ_TRANSACTION_PREFIX .. "(.+)";
			local iStart, iEnd, szCapture = string.find(szInfo, pattern);
			RDKP_Process_Transaction_Historic(szCapture);
			return true;
		else
			-- At this point, it doesn't match anything else, and there is a potential problem.  
			-- Who knows what this string should be.
			-- Log it?
			RDKP_Debug("Unknown transmission:" .. szInfo);
			-- It used to be a transaction w/o a header.
			RDKP_Process_Transaction_Historic(szCapture);
		end
		
	else
		-- Invalid info there shouldn't be any other channel paths yet.
		RDKP_Print("WARNING: " .. szSource .. " is transmitting on " .. szType .. " Addon Channel.")
		return;
	end
end

function RDKP_RecieveRaidID(szRaidInfo)
	RDKP_Debug("RDKP_RecieveRaidID(" .. szRaidInfo .. ")" );
	-- And Incoming packet is Stating that someone else has a particular RaidID.
	local szPattern = "(.+)".. RDKP_SZ_FIELD_DELIMITER .."(%d+)"
	local iStart, iEnd, szRaidID, iRaidLength = string.find(szRaidInfo, szPattern);	
	RDKP_Debug("RecieveRaidInfo:", szRaidID, iRaidLength);
	if (RDKP_Table_Raid_Register[szRaidID] == nil) then
		-- Request that Raid Info.
		RDKP_Table_Raid_Register[szRaidID] = {};
		RDKP_Guild_SendInfo(RDKP_SZ_REQUESTRAID_PREFIX .. szRaidID);
	elseif (table.getn(RDKP_Table_Raid_Register[szRaidID]) < tonumber(iRaidLength)) then
		-- Request that Raid Info and overwrite what we have.
		RDKP_Debug("RDKP_Table_Raid_Register Length = " .. table.getn(RDKP_Table_Raid_Register[szRaidID]))
		RDKP_Table_Raid_Register[szRaidID] = {};
		RDKP_Guild_SendInfo(RDKP_SZ_REQUESTRAID_PREFIX .. szRaidID);
	end
end

